#include "AverField.h"

#include <assert.h>
#include <iostream>

AverField::AverField(const Field& f, float(*convert)(const Cell&), size_t avg_radius, Rectangle area)
{
	int iar = (int)avg_radius;
	Area = area;
	assert(avg_radius >= 0);
	assert(avg_radius < f.getWidth() && avg_radius < f.getHeight());
	Averaged = new float[Area.Width* Area.Height];

	float area_size = (1+avg_radius*2);
	area_size *= area_size;

	for(int i=0; i<Area.Height; i++)
	{
		//std::cout << "Averaging line " << i << " of " << Area.Height << std::endl;
		for(int j=0; j<Area.Width; j++)
		{
			Averaged[i*Area.Width+j] = 0;

			for(int ii = -iar; ii <= iar; ii++)
				for(int jj = -iar; jj <= iar; jj++)
				{
			//		std::cout << "i=" << i << " j=" << j << " ii= " << ii << " ,jj=" << jj << ", state="
			//			<< f.getCellPeriodic(j+jj, i+ii).State << ", dens=" << convert(f.getCellPeriodic(j+jj, i+ii)) << std::endl;
					Averaged[i*Area.Width+j] +=
						convert(f.getCellPeriodic(Area.Left+j+jj, Area.Top+i+ii));
				}

			Averaged[i*Area.Width+j] /= area_size;
		}
	}
}
	/*
{
	assert(avg_radius < f.getWidth() && avg_radius < f.getHeight());
	// Algorithm:
	// """""""""
	// Create array of larger size for areas
	// Fill the array circularly
	// Calculate areas
	// Allocate Averaged array & calculate its data
	// dispose areas array;
	size_t areas_width = f.getWidth() + 2*avg_radius + 1;
	size_t areas_height = f.getHeight() + 2*avg_radius + 1;
	double *areas = new double[areas_width*areas_height];

	for(unsigned int i=0; i<areas_height; i++)
	{
		for(unsigned int j=0; j<areas_width; j++)
		{
			unsigned int row = (i - avg_radius + f.getHeight()) % f.getHeight();
			unsigned int col = (j - avg_radius + f.getWidth()) % f.getWidth();
			areas[i*areas_width+j] = convert(f.at(col, row));
		}
	}


	// Calculate areas
	for(unsigned int i=1; i<areas_height; i++)
		areas[i*areas_width] += areas[(i-1)*areas_width];

	for(unsigned int j=1; j<areas_width; j++)
		areas[j] += areas[j-1];

	for(unsigned int i=1; i<areas_height; i++)
	{
		for(unsigned int j=1; j<areas_width; j++)
		{
			areas[i*areas_width+j] +=
				areas[i*areas_width+j-1] +
				areas[(i-1)*areas_width+j] -
				areas[(i-1)*areas_width+j-1];
		}
	}

	Averaged = new float[f.getWidth()*f.getHeight()];

	float area_size = (1+2*avg_radius)*(1+2*avg_radius);

	for(unsigned int i=0; i<f.getHeight(); i++)
	{
		for(unsigned int j=0; j<f.getWidth(); j++)
		{
			unsigned int i1 = i+1+avg_radius*2,
						 j1 = j+1+avg_radius*2;
			Averaged[i*f.getWidth()+j] = (areas[i1*areas_width+j1]+areas[i*areas_width+j]-areas[i*areas_width+j1]-areas[i1*areas_width+j])/area_size;
			if(Averaged[i*f.getWidth()+j]==0)
				abort();
		}
	}

	delete [] areas;
}
*/
