/*
 * BoundaryProcessor.cpp
 *
 *  Created on: Feb 4, 2013
 *      Author: perepelkin
 */

#include "BoundaryProcessor.h"
#include "NormalProcessor.h"
#include <assert.h>

void BoundaryProcessor::process(const Field& src, Field& dest, int, int rank, size_t)
{
	if(rank != 0)
		return;
	assert(src.getWidth() == dest.getWidth());
	assert(src.getHeight() == dest.getHeight());
	assert(src.getWidth() >= Area.getRight());
	assert(src.getHeight() >= Area.getBottom());

	for(uint row=Area.getTop(); row<Area.getBottom(); row++)
		for(uint col=Area.getLeft(); col<Area.getRight(); col++)
		{
			NormalProcessor::shiftRule(
					src.getCellPeriodic(col, row),
					src.getCellPeriodic(col+1, row),
					src.getCellPeriodic(col, row+1),
					src.getCellPeriodic(col-1, row),
					src.getCellPeriodic(col, row-1), dest.at(col, row));

			dest.at(col, row) = bounce_rule(dest.at(col, row));
		}
}

Cell BoundaryProcessor::bounce_rule(const Cell& c)
{
	Cell result(0);
	result.setRight(c.getLeft());
	result.setLeft(c.getRight());
	result.setUp(c.getDown());
	result.setDown(c.getUp());

	return result;
}
