/*
 * Cell.h
 *
 *  Created on: Jan 31, 2013
 *      Author: perepelkin
 */

#ifndef CELL_H_
#define CELL_H_

#include <stdlib.h> // for size_t
#include <assert.h>

#define CELL_STATE unsigned char

class Cell {
public:
	static const CELL_STATE c_Move_mask;
	static const CELL_STATE c_Rest_particles_mask;

	CELL_STATE State;

	explicit Cell(CELL_STATE state=0);
inline unsigned char getRestMass() const
{
	return (State>>4)*2;
}

	unsigned char getMoveMass() const;
	unsigned char getMass() const;
	unsigned char getImpulse() const;

inline bool getRight() const { return State & 0x01; }
inline bool getDown() const { return State & 0x02; }
inline bool getLeft() const { return State & 0x04; }
inline bool getUp() const { return State & 0x08; }

inline void setRestMass(size_t rest_mass)
{
	assert(rest_mass%2==0);
	assert(rest_mass<=30);

	rest_mass = (rest_mass/2) << 4;
	State &= 0x0F;
	State += rest_mass;
}

inline void setRight(bool particle)
{
	if(particle)
		State |= 0x01;
	else
		State &= ~(0x01);
}

inline void setDown(bool particle)
{
	if(particle)
		State |= 0x02;
	else
		State &= ~(0x02);
}

inline void setLeft(bool particle)
{
	if(particle)
		State |= 0x04;
	else
		State &= ~(0x04);
}

inline void setUp(bool particle)
{
	if(particle)
		State |= 0x08;
	else
		State &= ~(0x08);
}


	bool operator==(const Cell&) const;
	bool operator!=(const Cell&) const;


};

#endif /* CELL_H_ */
