/*
 * Cell_test.cpp
 *
 *  Created on: Jan 31, 2013
 *      Author: perepelkin
 */

#include "Cell.h"

#include <iostream>
#include <assert.h>

#define IMPLIES(a, b) ((!(a)) || (b))
#define EQUALS(a,b) (IMPLIES(a,b) && IMPLIES(b,a))

static void test_Cell_consts();
static void test_Cell_Cell();
static void test_Cell_getRestMass();
static void test_Cell_getMoveMass();
static void test_Cell_getMass();
static void test_Cell_getImpulse();
static void test_Cell_operators_eq();
static void test_Cell_directions();
static void test_Cell_setRestMass();

void test_Cell()
{
	test_Cell_consts();
	test_Cell_Cell();
	test_Cell_getRestMass();
	test_Cell_getMoveMass();
	test_Cell_getMass();
	test_Cell_getImpulse();
	test_Cell_operators_eq();
	test_Cell_directions();
	test_Cell_setRestMass();
}

static void test_Cell_consts()
{
	assert(Cell::c_Move_mask == (CELL_STATE)0x0F);
	assert(Cell::c_Rest_particles_mask == 0xF0);
}


static void test_Cell_Cell()
{
	for(int c=0; c<256; c++)
		assert(Cell(c).State == (CELL_STATE)c);
}

static void test_Cell_getRestMass()
{
	assert(Cell(0).getRestMass() == 0);

	for(int c=0; c<256; c++)
		assert(Cell(c).getRestMass() == Cell(c&0xF0).getRestMass());

	for(int c=0; c<16; c++)
		assert(Cell(c<<4).getRestMass() == c<<1);

	assert(Cell(0x34).getRestMass() == 6);
}

static void test_Cell_getMoveMass()
{
	assert(Cell(0).getMoveMass() == 0);

	for(int c=0; c<256; c++)
		assert(Cell(c).getMoveMass() == Cell(c&0x0F).getMoveMass());

	for(char c=0; c<16; c++)
	{
		unsigned int mass=0;
		char c_tmp = c;
		while(c_tmp>0)
		{
			mass += (c_tmp&1)? 1: 0;
			c_tmp >>= 1;
		}
		assert(Cell(c).getMoveMass() == mass);
	}
}

static void test_Cell_getMass()
{
	assert(Cell(0).getMass()==0);

	for(int c=0; c<256; c++)
	{
		assert(Cell(c).getMass() == Cell(c).getRestMass() + Cell(c).getMoveMass());
		assert(Cell(c).getMass() <= 34);
	}

	assert(Cell(255).getMass()==34);
}

static void test_Cell_getImpulse()
{
	assert(Cell(0).getImpulse() == 0);
	assert(Cell(255).getImpulse() == 0);
	assert(Cell(0x03).getImpulse() == 0x03);
	assert(Cell(0x07).getImpulse() == 0x02);

	for(int c=0; c<256; c++)
	{
		Cell cell(c);
		assert(IMPLIES(cell.getImpulse(), cell.getMoveMass()));
		Cell move(c);
		move.setRestMass(0);
		assert(move.getImpulse() == cell.getImpulse());
	}
}

static void test_Cell_operators_eq()
{
	for(int i=0; i<1000; i++)
	{
		Cell c1((i+i*i)%256), c2(100+i);
		assert(EQUALS(c1==c2, c1.State==c2.State));
		assert(EQUALS(c1==c2, !(c1!=c2)));
	}
}

static void test_Cell_directions()
{
	assert(Cell(0).getRight() == false);
	assert(Cell(0).getDown() == false);
	assert(Cell(0).getLeft() == false);
	assert(Cell(0).getUp() == false);

	for(int i=0; i<256; i++)
	{
		Cell c(i);
		assert(EQUALS(c.getRight(), c.State&0x01));
		assert(EQUALS(c.getDown(), c.State&0x02));
		assert(EQUALS(c.getLeft(), c.State&0x04));
		assert(EQUALS(c.getUp(), c.State&0x08));

		Cell c1;
		c1 = c; c1.setRight(!c.getRight());
		assert((c1.State ^ c.State) == 0x01);
		c1 = c; c1.setDown(!c.getDown());
		assert((c1.State ^ c.State) == 0x02);
		c1 = c; c1.setLeft(!c.getLeft());
		assert((c1.State ^ c.State) == 0x04);
		c1 = c; c1.setUp(!c.getUp());
		assert((c1.State ^ c.State) == 0x08);
	}
}

static void test_Cell_setRestMass()
{
	for(int i=0; i<256; i++)
		for(int j=0; j<16; j++)
		{
			Cell c(i);
			c.setRestMass(j*2);
			assert(c.getRestMass()==j*2);
		}
}
























