/*
 * Field_test.cpp
 *
 *  Created on: Jan 31, 2013
 *      Author: perepelkin
 */

#include "Field.h"
#include <assert.h>

#define assert_throws(expression, exc_type)\
{\
	bool exc_thrown=false;\
	try\
	{\
		expression;\
	}\
	catch(exc_type)\
	{\
		exc_thrown=true;\
	}\
	assert(exc_thrown);\
}

void test_Field_Field();
void test_Field_getWidth();
void test_Field_getHeight();
void test_Field_at();
void test_Field_getCellPeriodic();

void test_Field()
{
	test_Field_Field();
	test_Field_getWidth();
	test_Field_getHeight();
	test_Field_at();
	test_Field_getCellPeriodic();
}

static Cell test_init(uint col, uint row, size_t width, size_t height)
{
	return Cell((col+row) % 256);
}

void test_Field_Field()
{
	assert_throws(Field(0, 0), Field::Exception&);
	assert_throws(Field(103, 0), Field::Exception&);
	assert_throws(Field(0, 30), Field::Exception&);
	Field(10, 10);
	Field(100, 33);

	Field f(10, 10, test_init);
	for(uint row=0; row<f.getHeight(); row++)
		for(uint col=0; col<f.getWidth(); col++)
		{
			assert(f.at(col, row)==test_init(col, row, f.getWidth(), f.getHeight()));
			assert(test_init(col, row, f.getWidth(), f.getHeight())==Cell((col+row)%256));
		}
}

void test_Field_getWidth()
{
	for(size_t width=1; width<10; width++)
		for(size_t height=1; height<10; height++)
			assert(Field(width, height).getWidth() == width);
}

void test_Field_getHeight()
{
	for(size_t width=1; width<10; width++)
			for(size_t height=1; height<10; height++)
				assert(Field(width, height).getHeight() == height);
}

void test_Field_at()
{
	size_t width = 10, height = 10;
	Field f(width, height);

	for(uint col=0; col<height; col++)
		for(uint row=0; row<width; row++)
		{
			Cell c(col+row*col);
			f.at(col, row) = c;
			assert(f.at(col, row) == c);
			c.State++;
			assert(f.at(col, row) != c);
		}
}

void test_Field_getCellPeriodic()
{
	size_t width = 10, height = 10;
	Field f(width, height);

	for(int col=-20; col<(int)height+20; col++)
		for(int row=-20; row<(int)width+20; row++)
		{
			int ucol = col, urow = row;
			while(ucol < 0) ucol += width;
			while(ucol >= (int)width) ucol -= width;
			while(urow < 0) urow += height;
			while(urow >= (int)height) urow -= height;

			Cell c(ucol+urow*ucol);
			f.at(ucol, urow) = c;
			assert(f.getCellPeriodic(col, row) == c);
			assert(f.at(ucol, urow) == f.getCellPeriodic(col, row));
			c.State++;
			assert(f.at(ucol, urow) != c);
			assert(f.getCellPeriodic(col, row) != c);
		}

}













