/*
 * NormalProcessor.cpp
 *
 *  Created on: Feb 4, 2013
 *      Author: perepelkin
 */

#include "NormalProcessor.h"
#include <assert.h>
#include <iostream>
#include <string.h>
#include <stdio.h>

NormalProcessor::~NormalProcessor()
{
	if(!CheckProbabilities)
		return;

	std::cout << "Probabilities check mode was on, displaying actual rules probabilities" << std::endl;
	for(int i=0; i<EndState; i++)
	{
		size_t total=0;
		bool only_trivial=true;
		for(int j=0; j<EndState; j++)
		{
			total += RulesLog[i*EndState+j];
			if(j!=i && RulesLog[i*EndState+j] !=0)
				only_trivial = false;
		}

		if(only_trivial)
			continue;

		std::cout << i << ":";
		for(int j=0; j<EndState; j++)
		{
			if(RulesLog[i*EndState+j] != 0)
				std::cout << " " << RulesLog[i*EndState+j] * 1.0 / total << "~" << Transitions[i][0].first << "_" << j;
		}
		std::cout << " -- total: " << total << std::endl;
	}

	delete RulesLog;
	delete [] Transitions;
}

static void save_prob_table(const std::vector<std::pair<float, int> >* table, size_t count, const std::string& filename)
{
	FILE* fl = fopen(filename.c_str(), "w");
	if(!fl)
	{
		std::cerr << "Error: Failed to save probabilities table to file " << filename << std::endl;
		abort();
	}

	for(int i=0; i<count; i++)
	{
		float rest_prob = 1.0f;

		fprintf(fl, "%d:", i);

		for(std::vector<std::pair<float, int> >::const_iterator ii = table[i].begin(); ii != table[i].end(); ii++)
		{
			fprintf(fl, "  %f_%d", ii->first, ii->second);
			rest_prob -= ii->first;
		}
		fprintf(fl, "  %f_%d\n", rest_prob, i);
	}
	fclose(fl);
}

NormalProcessor::NormalProcessor(unsigned int end_state, Rectangle area, bool check_probs, const std::string& save_table, bool no_rp_create)
{
	NoRPCreate = no_rp_create;
	RulesLog = NULL;
	SaveTablePath = save_table;
	CheckProbabilities = check_probs;
	Area = area;
	EndState = end_state;
	Transitions = new std::vector<std::pair<float, int> >[EndState];
	for(unsigned int i=0; i<EndState; i++)
	{
		std::vector<int> states;

		for(unsigned int j=0; j<EndState; j++)
		{
			if(i != j
					&& Cell(i).getMass() == Cell(j).getMass()
					&& Cell(i).getImpulse() == Cell(j).getImpulse()
					&& ((!NoRPCreate) || (Cell(i).getRestMass() >= Cell(j).getRestMass())))
			{
				states.push_back(j);
			}
		}
		if(states.empty()==false)
		{
			float prob = 1.0/(states.size()+1);
			float sum_prob = prob;
			for(unsigned int j=0; j<states.size(); j++)
			{
				Transitions[i].push_back(std::make_pair(prob, states[j]));
			}
		}
	}
	if(SaveTablePath != "")
		save_prob_table(Transitions, EndState, SaveTablePath);
	if(check_probs)
	{
		RulesLog = new size_t[end_state*end_state];
		memset(RulesLog, 0, sizeof(size_t)*end_state*end_state);
	}
}

void NormalProcessor::process(const Field& src, Field& dest, int iter, int rank, size_t size)
{
	assert(src.getWidth() == dest.getWidth());
	assert(src.getHeight() == dest.getHeight());
	assert(src.getWidth() >= Area.getRight());
	assert(src.getHeight() >= Area.getBottom());

	uint row_start = Area.getTop()+1 + rank * (Area.Height-2) / size;
	uint row_end = Area.getTop()+1 + (rank+1) * (Area.Height-2) / size;
	for(uint row=row_start; row<row_end; row++)
		for(uint col=Area.getLeft()+1; col<Area.getRight()-1; col++)
		{
			assert(src.at(col, row).State < EndState); // or maybe just leave it unchanged?
			shiftRule(
					src.at(col, row),
					src.at(col+1, row),
					src.at(col, row+1),
					src.at(col-1, row),
					src.at(col, row-1), dest.at(col, row));
			int old_state=dest.at(col, row).State;
			dest.at(col, row).State = collapseRule(dest.at(col, row));
			int new_state = dest.at(col, row).State;
			if(CheckProbabilities)
			{
				assert(old_state < EndState && new_state < EndState);
				RulesLog[old_state*EndState+new_state]++;
			}
		}
	if(rank==0)
	{
		for(uint col=Area.getLeft()+1; col<Area.getRight()-1; col++)
		{
			uint row = Area.getTop();
			if(row<Area.getBottom())
			{
				assert(src.at(col, row).State < EndState); // or maybe just leave it unchanged?
				shiftRule(
						src.getCellPeriodic(col, row),
						src.getCellPeriodic(col+1, row),
						src.getCellPeriodic(col, row+1),
						src.getCellPeriodic(col-1, row),
						src.getCellPeriodic(col, row-1), dest.at(col, row));
				int old_state=dest.at(col, row).State;
				dest.at(col, row).State = collapseRule(dest.at(col, row));
				int new_state = dest.at(col, row).State;
				if(CheckProbabilities)
				{
					assert(old_state < EndState && new_state < EndState);
					RulesLog[old_state*EndState+new_state]++;
				}
			}
			row = Area.getBottom()-1;
			if(row>=Area.getTop())
			{
				assert(src.at(col, row).State < EndState); // or maybe just leave it unchanged?
				shiftRule(
						src.getCellPeriodic(col, row),
						src.getCellPeriodic(col+1, row),
						src.getCellPeriodic(col, row+1),
						src.getCellPeriodic(col-1, row),
						src.getCellPeriodic(col, row-1), dest.at(col, row));
				int old_state=dest.at(col, row).State;
				dest.at(col, row).State = collapseRule(dest.at(col, row));
				int new_state = dest.at(col, row).State;
				if(CheckProbabilities)
				{
					assert(old_state < EndState && new_state < EndState);
					RulesLog[old_state*EndState+new_state]++;
				}
			}
		}
		for(uint row=Area.getTop(); row<Area.getBottom(); row++)
		{
			uint col = Area.getLeft();
			if(col<Area.getRight())
			{
				assert(src.at(col, row).State < EndState); // or maybe just leave it unchanged?
				shiftRule(
						src.getCellPeriodic(col, row),
						src.getCellPeriodic(col+1, row),
						src.getCellPeriodic(col, row+1),
						src.getCellPeriodic(col-1, row),
						src.getCellPeriodic(col, row-1), dest.at(col, row));
				int old_state=dest.at(col, row).State;
				dest.at(col, row).State = collapseRule(dest.at(col, row));
				int new_state = dest.at(col, row).State;
				if(CheckProbabilities)
				{
					assert(old_state < EndState && new_state < EndState);
					RulesLog[old_state*EndState+new_state]++;
				}
			}
			col = Area.getRight()-1;
			if(col>=Area.getLeft())
			{
				assert(src.at(col, row).State < EndState); // or maybe just leave it unchanged?
				shiftRule(
						src.getCellPeriodic(col, row),
						src.getCellPeriodic(col+1, row),
						src.getCellPeriodic(col, row+1),
						src.getCellPeriodic(col-1, row),
						src.getCellPeriodic(col, row-1), dest.at(col, row));
				int old_state=dest.at(col, row).State;
				dest.at(col, row).State = collapseRule(dest.at(col, row));
				int new_state = dest.at(col, row).State;
				if(CheckProbabilities)
				{
					assert(old_state < EndState && new_state < EndState);
					RulesLog[old_state*EndState+new_state]++;
				}
			}
		}
	}
}
/*
Cell NormalProcessor::shiftRule(const Cell& middle, const Cell& right, const Cell& down, const Cell& left, const Cell& up)
{
	Cell result;

	result.setRestMass(middle.getRestMass());
	result.setRight(left.getRight());
	result.setDown(up.getDown());
	result.setLeft(right.getLeft());
	result.setUp(down.getUp());

	return result;
}

Cell NormalProcessor::collapseRule(const Cell& c)
{
	int index = c.State;
	assert(index < EndState);

	if(Transitions[index].empty())
		return c;

	//int count = 1+Transitions[index].size();

	double r = rand()%100000/100000.0;

	unsigned int candidate=0;
	while(r>Transitions[index][candidate].first && candidate<Transitions[index].size())
	{
		r -= Transitions[index][candidate].first;
		candidate++;
	}

	if(candidate < Transitions[index].size())
		return Transitions[index][candidate].second;
	else
		return c.State;
}
*/






