/*
 * RulesTableProcessor.cpp
 *
 *  Created on: Feb 20, 2013
 *      Author: perepelkin
 */

#include "NormalProcessor.h"
#include "RulesTableProcessor.h"
#include <assert.h>
#include <iostream>
#include <string.h>
#include <stdio.h>
#include "json.h"
#include <sstream>

RulesTableProcessor::~RulesTableProcessor()
{
	delete [] Transitions;
}

std::string loadFileToString(const char*);

std::vector<std::string> split(const std::string& str, char c)
{
	std::string s=str;
	std::vector<std::string> res;

	while(s.find(c) != std::string::npos)
	{
		res.push_back(s.substr(0, s.find(c)));
		s = s.substr(s.find(c)+1);
	}

	if(s.empty()==false)
		res.push_back(s);

	return res;
}

std::string get_from(const std::string& line)
{
	return line.substr(0, line.find(':'));
}

int bit_to_int(const std::string bits)
{
	int res=0;
	int base=1;
	for(unsigned int i=0; i<bits.size(); i++)
	{
		char bit = bits[bits.size()-1-i];
		if(!(bit=='0' || bit=='1'))
		{
			std::cerr << "Not a binary number: " << bits << std::endl;
			abort();
		}
		if(bit=='1')
			res += base;
		base *= 2;
	}
	return res;
}

void trim(std::string& s)
{
	while(s.empty()==false && s[0]==' ') s=s.substr(1);
	while(s.empty()==false && s[s.size()-1]==' ') s=s.substr(0, s.size()-1);
}

int get_bits(std::string& line)
{
	while(line.empty()==false && line[0]==' ') line = line.substr(1);

	std::string bits="";

	while(line.empty()==false && (line[0]=='0' || line[0]=='1'))
	{
		bits += line[0];
		line = line.substr(1);
	}

	trim(line);

	return bit_to_int(bits);
}

void drop(std::string& s, char c)
{
	if(s.empty() || s[0]!=c)
	{
		std::cerr << "Format error" << std::endl;
		abort();
	}

	s = s.substr(1);
	trim(s);
}

float get_float(std::string& s)
{
	std::string f;
	while(s.empty()==false && ((s[0]>='0' && s[0]<='9') || s[0]=='.'))
	{
		f += s[0];
		s = s.substr(1);
	}
	std::istringstream is(f);
	float res;
	is >> res;
	trim(s);
	return res;
}

std::pair<float, int> get_rule_pair(std::string& s)
{
	float prob = get_float(s);
	int state = get_bits(s);
	return std::make_pair(prob, state);
}

std::vector<std::pair<float, int> > get_tos(const std::string& line)
{
	std::vector<std::pair<float, int> > result;

	std::string rest = line;
	trim(rest);

	int from = get_bits(rest);
	drop(rest, ':');

	while(rest.empty()==false)
	{
		if(result.empty()==false)
			drop(rest, ',');
		result.push_back(get_rule_pair(rest));
	}

	return result;
}

RulesTableProcessor::RulesTableProcessor(const std::string& file, Rectangle area, size_t end_state)
{
	Area = area;
	EndState = end_state;

	std::string data = loadFileToString(file.c_str());

	std::vector<std::string> lines = split(data, '\n');

	Transitions = new std::vector<std::pair<float, int> >[EndState];

	for(std::vector<std::string>::iterator it = lines.begin(); it != lines.end(); it++)
	{
		std::string sfrom = get_from(*it);
		int from = bit_to_int(sfrom);
		if(Transitions[from].empty()==false)
		{
			std::cerr << "Duplicate rules set for state: " << sfrom << " (" << from << "/" << EndState << "), rules count: " << Transitions[from].size() << std::endl;
			abort();
		}
		Transitions[from] = get_tos(*it);
		std::cout << "Adding rules for " << sfrom << " (" << from << ")" << std::endl;
	}
}

void RulesTableProcessor::process(const Field& src, Field& dest, int iter, int rank, size_t size)
{
	assert(src.getWidth() == dest.getWidth());
	assert(src.getHeight() == dest.getHeight());
	assert(src.getWidth() >= Area.getRight());
	assert(src.getHeight() >= Area.getBottom());

	uint row_start = Area.getTop()+1 + rank * (Area.Height-2) / size;
	uint row_end = Area.getTop()+1 + (rank+1) * (Area.Height-2) / size;
	for(uint row=row_start; row<row_end; row++)
		for(uint col=Area.getLeft()+1; col<Area.getRight()-1; col++)
		{
			NormalProcessor::shiftRule(
					src.at(col, row),
					src.at(col+1, row),
					src.at(col, row+1),
					src.at(col-1, row),
					src.at(col, row-1), dest.at(col, row));
			int old_state=dest.at(col, row).State;
			dest.at(col, row).State = collapseRule(dest.at(col, row));
			int new_state = dest.at(col, row).State;
		}
	if(rank==0)
	{
		for(uint col=Area.getLeft()+1; col<Area.getRight()-1; col++)
		{
			uint row = Area.getTop();
			if(row<Area.getBottom())
			{
				NormalProcessor::shiftRule(
						src.getCellPeriodic(col, row),
						src.getCellPeriodic(col+1, row),
						src.getCellPeriodic(col, row+1),
						src.getCellPeriodic(col-1, row),
						src.getCellPeriodic(col, row-1), dest.at(col, row));
				int old_state=dest.at(col, row).State;
				dest.at(col, row).State = collapseRule(dest.at(col, row));
				int new_state = dest.at(col, row).State;
			}
			row = Area.getBottom()-1;
			if(row>=Area.getTop())
			{
				NormalProcessor::shiftRule(
						src.getCellPeriodic(col, row),
						src.getCellPeriodic(col+1, row),
						src.getCellPeriodic(col, row+1),
						src.getCellPeriodic(col-1, row),
						src.getCellPeriodic(col, row-1), dest.at(col, row));
				int old_state=dest.at(col, row).State;
				dest.at(col, row).State = collapseRule(dest.at(col, row));
				int new_state = dest.at(col, row).State;
			}
		}
		for(uint row=Area.getTop(); row<Area.getBottom(); row++)
		{
			uint col = Area.getLeft();
			if(col<Area.getRight())
			{
				NormalProcessor::shiftRule(
						src.getCellPeriodic(col, row),
						src.getCellPeriodic(col+1, row),
						src.getCellPeriodic(col, row+1),
						src.getCellPeriodic(col-1, row),
						src.getCellPeriodic(col, row-1), dest.at(col, row));
				int old_state=dest.at(col, row).State;
				dest.at(col, row).State = collapseRule(dest.at(col, row));
				int new_state = dest.at(col, row).State;
			}
			col = Area.getRight()-1;
			if(col>=Area.getLeft())
			{
				NormalProcessor::shiftRule(
						src.getCellPeriodic(col, row),
						src.getCellPeriodic(col+1, row),
						src.getCellPeriodic(col, row+1),
						src.getCellPeriodic(col-1, row),
						src.getCellPeriodic(col, row-1), dest.at(col, row));
				int old_state=dest.at(col, row).State;
				dest.at(col, row).State = collapseRule(dest.at(col, row));
				int new_state = dest.at(col, row).State;
			}
		}
	}
}
/*
Cell RulesTableProcessor::shiftRule(const Cell& middle, const Cell& right, const Cell& down, const Cell& left, const Cell& up)
{
	Cell result;

	result.setRestMass(middle.getRestMass());
	result.setRight(left.getRight());
	result.setDown(up.getDown());
	result.setLeft(right.getLeft());
	result.setUp(down.getUp());

	return result;
}

Cell RulesTableProcessor::collapseRule(const Cell& c)
{
	int index = c.State;
	assert(index < EndState);

	if(Transitions[index].empty())
		return c;

	//int count = 1+Transitions[index].size();

	double r = rand()%100000/100000.0;

	unsigned int candidate=0;
	while(r>Transitions[index][candidate].first && candidate<Transitions[index].size())
	{
		r -= Transitions[index][candidate].first;
		candidate++;
	}

	if(candidate < Transitions[index].size())
		return Transitions[index][candidate].second;
	else
		return c.State;
}
*/






