/*
 * SourceProcessor.cpp
 *
 *  Created on: Feb 4, 2013
 *      Author: perepelkin
 */

#include "SourceProcessor.h"
#include <iostream>
#include <assert.h>

static CELL_STATE random_state(const std::vector<std::pair<float, int> >& states)
{
	assert(states.empty() == false);
	float r = random()%1000000/1000000.0;
	int i=0;

	while(r > states[i].first && i < states.size())
	{
		r -= states[i].first;
		i++;
	}

	if(i <= states.size())
		return (states[i].second);
	else
		return (states[states.size()-1].second);
}

void SourceProcessor::process(const Field& src, Field& dest, int iter, int rank, size_t size)
{
	if(rank!=0)
		return;
	bool active = false;
	for(int i=0; i<Intervals.size(); i++)
		if(iter >= Intervals[i].first && iter <= Intervals[i].second)
		{
			active=true;
			break;
		}

	if(!active)
		return;

	assert(src.getWidth() == dest.getWidth());
	assert(src.getHeight() == dest.getHeight());
	assert(src.getWidth() >= Area.getRight());
	assert(src.getHeight() >= Area.getBottom());

	for(uint row=Area.getTop(); row<Area.getBottom(); row++)
		for(uint col=Area.getLeft(); col<Area.getRight(); col++)
		{
			dest.at(col, row).State = random_state(States);
		}
}

void CircleSourceProcessor::process(const Field& src, Field& dest, int iter,int rank, size_t size)
{
	if(rank != 0)
		return;
	bool active = false;
	for(int i=0; i<Intervals.size(); i++)
		if(iter >= Intervals[i].first && iter <= Intervals[i].second)
		{
			active=true;
			break;
		}

	if(!active)
		return;

	assert(src.getWidth() == dest.getWidth());
	assert(src.getHeight() == dest.getHeight());
	assert(X>=R && Y>=R && X+R<src.getWidth() && Y+R<src.getHeight());

	for(uint row=Y-R; row<=Y+R; row++)
		for(uint col=X-R; col<X+R; col++)
			if((row-Y)*(row-Y)+(col-X)*(col-X)<=R*R)
			{
				dest.at(col, row).State = random_state(States);
			}
}

void RandomSourceProcessor::process(const Field& src, Field& dest, int iter, int rank, size_t size)
{
	if(rank != 0)
		return;
	bool active = false;
	for(int i=0; i<Intervals.size(); i++)
		if(iter >= Intervals[i].first && iter <= Intervals[i].second)
		{
			active=true;
			break;
		}

	if(!active)
		return;

	assert(src.getWidth() == dest.getWidth());
	assert(src.getHeight() == dest.getHeight());
	assert(src.getWidth() >= Area.getRight());
	assert(src.getHeight() >= Area.getBottom());

	for(uint row=Area.getTop(); row<Area.getBottom(); row++)
		for(uint col=Area.getLeft(); col<Area.getRight(); col++)
		{
			dest.at(col, row).State = (random()%EndState);
		}
	std::cout << std::endl;
}


