/*
 * Visualizer.cpp
 *
 *  Created on: Feb 3, 2013
 *      Author: perepelkin
 */

#include "Visualizer.h"

#include <assert.h>
#include <iostream>
#include <sstream>
#include <stdio.h>
#include "bmpfile.h"
#include "AverField.h"

float Visualizer::convert_to_density(const Cell& c)
{
	return c.getMass();
}

float Visualizer::convert_to_hvelosity(const Cell& c)
{
	return (c.getRight()? 1: 0) - (c.getLeft()? 1: 0);
}

float* createReducedVertical(AverField& avf, size_t width, size_t height, const Rectangle& area)
{
	float *res = new float[width];

	for(uint j=0; j<width; j++)
	{
		res[j] = 0;
		for(uint i=0; i<height; i++)
			if(i>=area.getTop() && i<area.getBottom())
				res[j] += avf.at(j, i) / area.Height;
	}

	return res;
}

void Visualizer::logToFile(const Field& SourceField, const std::string& filename, size_t avg_radius) const
{
	size_t width = SourceField.getWidth();
	size_t height = SourceField.getHeight();
	bool draw_density1d=false;
	bool draw_hvelosity1d=false;
	for(std::vector<std::string>::const_iterator it = ToVisualize.begin(); it != ToVisualize.end(); it++)
	{
		if(*it=="density1d")
			draw_density1d = true;
		else if(*it=="hvelosity1d")
			draw_hvelosity1d = true;
		else
		{
			std::cerr << "Error: unsupported type of text-logging: " << *it << std::endl;
			abort();
		}
	}
	AverField avf(SourceField, convert_to_density, avg_radius, Area);
	if(draw_density1d)
	{
		//float* dens = createConvertedArray(SourceField, convert_to_density);
		//calculateAreas(dens, width, height);
		//float* avg = createAveragedArray(dens, avg_radius, width, height);
		float* dens1d = createReducedVertical(avf, width, height, Area);

		FILE* f = fopen(filename.c_str(), "w");
		if(f==NULL)
		{
			std::ostringstream os;
			os << "Failed to open file " << filename << " for writing";
			throw Exception(os.str());
		}

		for(uint col=0; col<width; col++)
			if(col >= Area.getLeft() && col < Area.getRight())
		{
			fprintf(f, "%f\n", dens1d[col]);
		}
		fclose(f);
		//delete [] dens;
		//delete [] avg;
		delete [] dens1d;
	}
	if(draw_hvelosity1d)
	{
		//float* dens = createConvertedArray(SourceField, convert_to_hvelosity);
		//calculateAreas(dens, width, height);
		//float* avg = createAveragedArray(dens, avg_radius, width, height);
		float* dens1d = createReducedVertical(avf, width, height, Area);

		FILE* f = fopen(filename.c_str(), "w");
		if(f==NULL)
		{
			std::ostringstream os;
			os << "Failed to open file " << filename << " for writing";
			throw Exception(os.str());
		}

		for(uint col=0; col<width; col++)
			if(col >= Area.getLeft() && col < Area.getRight())
		{
			fprintf(f, "%f\n", dens1d[col]);
		}
		fclose(f);
		//delete [] dens;
		//delete [] avg;
		delete [] dens1d;
	}

}

void Visualizer::saveToBmp(const Field& SourceField, const std::string& filename, float(*convert)(const Cell&), size_t avg_radius) const
{
	size_t width = SourceField.getWidth();
	size_t height = SourceField.getHeight();
	//float* dens = createConvertedArray(SourceField, convert);
	//calculateAreas(dens, width, height);
	//float* avg = createAveragedArray(dens, avg_radius, width, height);
	AverField avf(SourceField, convert, avg_radius, Area);
	float* dens1d = createReducedVertical(avf, width, height, Area);

	bool draw_density2d=false;
	bool draw_density1d=false;

	for(std::vector<std::string>::const_iterator it = ToVisualize.begin(); it != ToVisualize.end(); it++)
		if(*it=="density2d")
			draw_density2d = true;
		else if(*it=="density1d")
			draw_density1d = true;
		else
		{
			std::cerr << "Error: unsupported type of visualization: " << *it << std::endl;
			abort();
		}

	bmpfile_t *bmp = bmp_create(Area.Width, Area.Height, 24);
	//std::cout << "bmp_create" << std::endl;
	if(bmp==NULL)
	{
		std::ostringstream os;
		os << "Failed to create image of size " << width << " x " << height;
		throw Exception(os.str());
	}
	if(draw_density2d)
		for(uint row=0; row<height; row++)
			for(uint col=0; col<width; col++)
				if(col >= Area.getLeft() && col < Area.getRight()
						&& row >= Area.getTop() && row < Area.getBottom())
			{
				int color = (int)(avf.at(col, row) * 255 / 5);
				if(color>255)
					color=255;
				rgb_pixel_t pixel = {color, color, color};
				bmp_set_pixel(bmp, col-Area.getLeft(), row-Area.getTop(), pixel);
			}

	if(draw_density1d)
		for(uint col=0; col<width; col++)
			if(col >= Area.getLeft() && col < Area.getRight())
		{
			int row = (int)(Area.Height*(1-dens1d[col]/5));
			if(row>=0 && row<(int)Area.Height)
			{
				rgb_pixel_t blue = {0, 0, 255};
				bmp_set_pixel(bmp, col-Area.getLeft(), row, blue);
			}
		}

	bool saved=bmp_save(bmp, filename.c_str());
	bmp_destroy(bmp);
	if(!saved)
	{
		std::ostringstream os;
		os << "Failed to save image to file " << filename;
		throw Exception(os.str());
	}

//	delete [] dens;
//	delete [] avg;
	delete [] dens1d;
}

float* Visualizer::createConvertedArray(const Field& field, ConverterFunc convert)
{
	size_t width = field.getWidth(), height = field.getHeight();

	float *result = new float[height * width];

	for(uint row=0; row<height; row++)
		for(uint col=0; col<width; col++)
			result[row*width+col] = convert(field.at(col, row));

	return result;
}

void Visualizer::calculateAreas(float* arr, size_t width, size_t height)
{
	assert(width>0 && height>0);

	for(uint row=1; row<height; row++)
		arr[row*width] += arr[(row-1)*width];

	for(uint col=1; col<width; col++)
		arr[col] += arr[col-1];

	for(uint row=1; row<height; row++)
		for(uint col=1; col<width; col++)
		{
			arr[row*width+col] +=
					arr[row*width+col-1] +
					arr[(row-1)*width+col] -
					arr[(row-1)*width+col-1];
		}
}

float* Visualizer::createAveragedArray(float* areas, size_t avg_radius, size_t arr_width, size_t arr_height)
{
	int width = (int)arr_width;
	int height = (int)arr_height;
	assert(width>0 && height>0);
	//assert(avg_radius>0);
	float *result = new float[width*height];
	float area_size = (1+2*avg_radius) * (1+2*avg_radius);

	for(int row=0; row<height; row++)
		for(int col=0; col<width; col++)
		{
			int row_start = row-avg_radius-1,
					row_end = row+avg_radius,
					col_start = col-avg_radius-1,
					col_end = col+avg_radius;
			if(row_start<0 || row_end>=height || col_start<0 || col_end>=width)
				result[row*width+col] = 0; // TODO: improve formula to consider torus neighbors
			else
				result[row*width+col] = (areas[row_end*width+col_end] + areas[row_start*width+col_start]
								- areas[row_start*width+col_end] - areas[row_end*width+col_start])
								/ area_size;
		}
	return result;
}





