/*
 * Visualizer.h
 *
 *  Created on: Feb 3, 2013
 *      Author: perepelkin
 */

#ifndef VISUALIZER_H_
#define VISUALIZER_H_

#include "Writer.h"
#include "Field.h"
#include <string>
#include <vector>
#include <stdio.h>
#include <iostream>
#include <assert.h>
#include <string.h>
#include "Rectangle.h"

class Visualizer : public Writer
{
public:
	enum WriterType
	{
		Image,
		Text
	};
private:
	WriterType Type;
	size_t Period;
	size_t AverageRadius;
	std::string SavePath;
	std::vector<std::string> ToVisualize;
	Rectangle Area;
public:
	class Exception
	{
		std::string Message;
	public:
		Exception(const std::string& message) : Message(message) {}
		const std::string& getMessage() const { return Message; }
	};
	Visualizer(size_t period, size_t avg_radius, WriterType type, const std::string& path, const std::vector<std::string>& to_visualize,
			const Rectangle& area)
	: Type(type), Period(period), AverageRadius(avg_radius), SavePath(path), ToVisualize(to_visualize), Area(area)
	{}

	virtual size_t getPeriod() const { return Period; }

	void saveToBmp(const Field& SourceField, const std::string& filename, float(*convert)(const Cell&), size_t avg_radius) const;
	void logToFile(const Field& SourceField, const std::string& filename, size_t avg_radius) const;

	void process(const Field& f, int iter)
	{
		char fname[100];
		if(SavePath.size() > 50)
			abort();
		sprintf(fname, "%s/iter%08d", SavePath.c_str(), iter);
		std::string filename = fname;
		if(Type==Image)
		{
			filename += ".bmp";
			saveToBmp(f, filename.c_str(), convert_to_density, AverageRadius);
		}
		else if(Type==Text)
		{
			assert(ToVisualize.size()==1);
			filename += ".txt";
			logToFile(f, filename, AverageRadius);
		}
		else
		{
			throw "Unknown writer type";
		}
	}

	static float convert_to_density(const Cell&);
	static float convert_to_hvelosity(const Cell&);

private:

	static float* createConvertedArray(const Field&, ConverterFunc);
	static float* createAveragedArray(float* areas, size_t avg_radius, size_t width, size_t height);
	static void calculateAreas(float* arr, size_t width, size_t height);

	friend void test_Visualizer_createConvertedArray();
	friend void test_Visualizer_calculateAreas();
};

#endif /* VISUALIZER_H_ */
