/*
 * Visualizer_test.cpp
 *
 *  Created on: Feb 3, 2013
 *      Author: perepelkin
 */

#include "Visualizer.h"

#include <assert.h>

void test_Visualizer_createConvertedArray();
void test_Visualizer_calculateAreas();

void test_Visualizer()
{
	test_Visualizer_createConvertedArray();
	test_Visualizer_calculateAreas();
}

static float test_convert(const Cell& c)
{
	return (float)c.State;
}

static Cell test_init(uint col, uint row, size_t width, size_t height)
{
	return Cell(rand()%256);
}

void test_Visualizer_createConvertedArray()
{
	Field f(10,10, test_init);

	float *converted = Visualizer::createConvertedArray(f, test_convert);

	for(uint row=0; row<f.getHeight(); row++)
		for(uint col=0; col<f.getWidth(); col++)
		{
			assert(converted[row*f.getWidth()+col] == test_convert(f.at(col, row)));
			assert(test_convert(f.at(col, row))==(float)(f.at(col, row).State));
		}

	delete [] converted;
}

static float calc_area(const float* array, uint col, uint row, size_t width)
{
	float sum=0;

	for(uint y=0; y <= row; y++)
		for(uint x=0; x <= col; x++)
			sum += array[y*width+x];
	return sum;
}

void test_Visualizer_calculateAreas()
{
	Field f(100, 200, test_init);

	float *density = Visualizer::createConvertedArray(f, test_convert);
	float *areas = Visualizer::createConvertedArray(f, test_convert);
	Visualizer::calculateAreas(areas, f.getWidth(), f.getHeight());

	assert(density[0] == areas[0]);
	float sum=0;
	for(uint col = 0; col < f.getWidth(); col++)
	{
		sum += density[col];
		assert(sum == areas[col]);
	}

	sum=0;
	for(uint row=0; row < f.getHeight(); row++)
	{
		sum += density[row*f.getWidth()];
		assert(sum == areas[row*f.getWidth()]);
	}

	assert(areas[1*f.getWidth() + 1] == calc_area(density, 1, 1, f.getWidth()));
	assert(areas[2*f.getWidth() + 1] == calc_area(density, 1, 2, f.getWidth()));
	assert(areas[1*f.getWidth() + 2] == calc_area(density, 2, 1, f.getWidth()));
	assert(areas[10*f.getWidth() + 20] == calc_area(density, 20, 10, f.getWidth()));

	delete [] density;
	delete [] areas;
}
