#ifndef WRITER_H_
#define WRITER_H_

#include "Field.h"

#include <iostream>
#include <stdio.h>
#include <stdlib.h>

class Writer
{
public:
	virtual ~Writer(){}

	virtual size_t getPeriod() const=0;
	virtual void process(const Field&, int iter)=0;
};

class IterWriter : public Writer
{
	size_t Period;
	size_t TotalIters;
public:
	IterWriter(size_t period, size_t total_iters) : Period(period), TotalIters(total_iters) {}
	virtual size_t getPeriod() const { return Period; }
	virtual void process(const Field&, int iter)
	{
		std::cout << "Iter: " << iter << " of " << TotalIters << " (" << iter*100.0/TotalIters << "% complete)" << std::endl;
	}
};

class DumpWriter : public Writer
{
	size_t Period;
	std::string SavePath;
public:
	DumpWriter(size_t period, const std::string& folder) : Period(period), SavePath(folder) {}
	virtual size_t getPeriod() const { return Period; }
	virtual void process(const Field& f, int iter)
	{
		char fname[100];
		if(SavePath.size() > 50)
			abort();
		sprintf(fname, "%s/iter%08d", SavePath.c_str(), iter);

		FILE* fd = fopen(fname, "wb");
		if(!fd)
		{
			std::cerr << "Failed to open file for writing: " << fname << std::endl;
			abort();
		}

		for(int i=0; i<f.getHeight(); i++)
			for(int j=0; j<f.getWidth(); j++)
				fwrite(&f.at(j, i).State, 1, 1, fd);

		fclose(fd);
	}
};

#endif
