#include <stdio.h>
#include <stdlib.h>
#include "bmpfile.h"

const char* prog_name;

void usage()
{
	printf("Usage:\n\t%s <dump file> <width> <height>\n\t%s <dump file> <width> <height> <x0> <y0> <x1> <y1>\n\n",
			prog_name, prog_name);
	abort();
}

int main(int argc, char** argv)
{
	prog_name = argv[0];
	if(argc!=4 && argc!=8)
		usage();

	const char* dump_file = argv[1];

	int width, height, x0, x1, y0, y1;

	if(sscanf(argv[2], "%d", &width) != 1) usage();
	if(sscanf(argv[3], "%d", &height) != 1) usage();

	if(width<=0 || height<=0)
		usage();

	if(argc==8)
	{
		if(sscanf(argv[4], "%d", &x0) != 1) usage();
		if(sscanf(argv[5], "%d", &y0) != 1) usage();
		if(sscanf(argv[6], "%d", &x1) != 1) usage();
		if(sscanf(argv[7], "%d", &y1) != 1) usage();
	}
	else
	{
		x0=0; y0=0; x1=width; y1=height;
	}

	if(x0<0 || x0>width || x1<0 || x1>width || x1<=x0) usage();
	if(y0<0 || y0>height || y1<0 || y1>height || y1<=y0) usage();

	bmpfile_t *bmp = bmp_create(x1-x0, y1-y0, 24);
	if(!bmp)
	{
		fprintf(stderr, "Failed to create BMP object of size %dx%d\n", x1-x0, y1-y0);
		abort();
	}

	FILE* fd = fopen(dump_file, "rb");
	if(!fd)
	{
		fprintf(stderr, "Failed to open file '%s' for reading\n", dump_file);
		abort();
	}

	for(int i=0; i<height; i++)
		for(int j=0; j<width; j++)
		{
			if(feof(fd))
			{
				fprintf(stderr, "Unexpected end of file at position (%d,%d). Wrong width/height?\n", j, i);
				abort();
			}
			char c;
			fread(&c, 1, 1, fd);
			if(i>=y0 && i<y1 && j>=x0 && j<x1)
			{
				int mass = c;
				mass >> 4;
				mass *= 2;
				if(c & 0x01) mass++;
				if(c & 0x02) mass++;
				if(c & 0x04) mass++;
				if(c & 0x08) mass++;

				int color = mass*51;
				if(color>255) color=255;

				rgb_pixel_t p = {color, color, color};
				bmp_set_pixel(bmp, j-x0, i-y0, p);
			}
		}
	fclose(fd);
	std::string s = dump_file;
	s += ".bmp";
	bool saved = bmp_save(bmp, s.c_str());
	if(!saved)
	{
		fprintf(stderr, "Failed to write BMP to file %s\n", s.c_str());
		abort();
	}
	bmp_destroy(bmp);
	return 0;
}
