/*
 * main.cpp
 *
 *  Created on: Jan 30, 2013
 *      Author: perepelkin
 */

#include <iostream>
#include <stdio.h>
#include <sys/time.h>
#include <assert.h>
#include <sstream>

#include "Field.h"
#include "Visualizer.h"
#include "NormalProcessor.h"
#include "BoundaryProcessor.h"
#include "SourceProcessor.h"
#include <omp.h>

void test_main();

long field_width, field_height, iters, end_state;
std::vector<Writer*> Writers;
std::vector<Processor*> Procs;

void load_configuration(const char* filename);

int main()
{
#ifndef NDEBUG
	test_main();
#endif

	load_configuration("model.txt");

	Field f(field_width, field_height), f1(field_width, field_height);

	struct timeval t0, t;
	gettimeofday(&t0, NULL);
//#pragma omp parallel
{
	int rank = omp_get_thread_num();
	int size = omp_get_num_threads();
	for(uint i=0; i<iters; i++)
	{
		for(std::vector<Processor*>::iterator it = Procs.begin(); it != Procs.end(); it++)
		if(i%2==0)
			(*it)->process(f, f1, i, rank, size);
		else
			(*it)->process(f1, f, i, rank, size);

#pragma omp barrier
		
		try
		{
		for(std::vector<Writer*>::iterator it = Writers.begin(); it != Writers.end(); it++)
			if(i%(*it)->getPeriod()==0)
			{
				if(i%2==0)
					(*it)->process(f1, i);
				else
					(*it)->process(f, i);
			}
		}catch(Visualizer::Exception& x)
		{
			std::cerr << "Visualizer error: " << x.getMessage() << std::endl;
			abort();
		}
		
	}
}
	gettimeofday(&t, NULL);

	double time = (t.tv_sec-t0.tv_sec)+0.000001*(t.tv_usec-t0.tv_usec);
	double MCPS = f.getWidth() * f.getHeight() * iters / time / 1000000.0;

	std::cout << "Total time: " << time << " sec., " << MCPS << " Mcells/sec." << std::endl;

	for(std::vector<Processor*>::iterator it = Procs.begin(); it != Procs.end(); it++)
		delete *it;
	Procs.clear();
	for(std::vector<Writer*>::iterator it = Writers.begin(); it != Writers.end(); it++)
		delete *it;
	Writers.clear();

	return 0;
}
