// CONFIGURATION

#define RP_BITS 1
float RP_PROB[RP_BITS] = {0.2};

// END OF CONFIGURATION

#include <stdio.h>
#include <stdlib.h>
int end_state;
int rp_states;

float A[256*256]={0.0};

void calc_end_states()
{
	int rp_bits = RP_BITS;
	rp_states=1;
	end_state = 16;
	while(rp_bits-->0)
	{
		end_state<<=1;
		rp_states<<=1;
	}
	printf("end_state=%d\n", end_state);
}

float get_prob_rp(int i, int j)
{
	float res = 1.0;
	int mask=0x01;
	for(int bit=0; bit<RP_BITS; bit++)
	{
		if((mask&i) != (mask&j))
			res *= RP_PROB[bit];
		else
			res *= (1-RP_PROB[bit]);
		mask <<=1;
	}
	return res;
}

int mass(int state)
{
	int res=(state>>4)*2;
	for(int i=0; i<4; i++)
	{
		if(state&1)
			res++;
		state>>=1;
	}
	return res;
}

int impulse(int mp)
{
	if((mp&0x05)==0x05)
		mp -= 0x05;
	if((mp&0x0a)==0x0a)
		mp -= 0x0a;
	return mp;
}

bool valid(int rp1, int mp1, int rp2, int mp2)
{
	int state1=(rp1<<4)+mp1,
		state2=(rp2<<4)+mp2;

	return (impulse(mp1)==impulse(mp2)) && (mass(state1)==mass(state2));
}

int set_prob(int rp1, int mp1, int rp2, int mp2, float p)
{
	int state1=(rp1<<4)+mp1,
		state2=(rp2<<4)+mp2;
	A[state1*256+state2] = p;
}

void calc_rules()
{
	for(int i=0; i<rp_states; i++)
	{
		for(int k=0; k<16; k++)
		{
			float norm_prob=0;
			for(int j=0; j<rp_states; j++)
			{
				float prob = get_prob_rp(i,j);
				int count=0;
				for(int l=0; l<16; l++)
					if(valid(i,k,j,l))
						count++;
				if(count!=0)
					norm_prob += prob;
			}
			for(int j=0; j<rp_states; j++)
			{
				float prob = get_prob_rp(i,j);
				int count=0;
				for(int l=0; l<16; l++)
					if(valid(i,k,j,l))
						count++;
				for(int l=0; l<16; l++)
					if(valid(i,k,j,l))
					{
						set_prob(i,k,j,l,prob/count/norm_prob);
					}
			}
		}
	}
}

void check_ballance()
{
	bool failed=false;
	int i, j;
	for(i=0; i<end_state; i++)
	{
		float sum=0;
		for(j=0; j<end_state; j++)
			sum += A[i*256+j];
		if(sum!=1)
		{
			printf("Horiz Ballance wrong for state %d: %f\n", i, sum);
			failed=true;
		};
	}
	for(j=0; j<end_state; j++)
	{
		float sum=0;
		for(i=0; i<end_state; i++)
			sum += A[i*256+j];
		if(sum!=1)
		{
			printf("Vert Ballance wrong for state %d: %f\n", j, sum);
			failed=true;
		};
	}
	if(failed)
	{
		printf("Ballance failed, aborting\n");
		//abort();
	}
}

void write_rules()
{
	printf("Rules:\n");
	for(int i=0; i<end_state; i++)
	{
		int count=0;
		for(int j=0; j<end_state; j++)
			if(A[i*256+j] != 0)
				count++;
		if(count>0)
		{
			int shown=0;
			printf("%d:", i);
			for(int j=0; j<end_state; j++)
				if(A[i*256+j] != 0)
					printf(" %f %d%s", A[i*256+j], j, ++shown==count? "": ",");
			printf("\n");
		}
	}
}

int main()
{
	calc_end_states();

	calc_rules();

	check_ballance();

	write_rules();

	return 0;
}
